import netgen.gui
from netgen.geom2d import unit_square
from ngsolve import *
import time
mesh = Mesh(unit_square.GenerateMesh(maxh=0.01))

geom_free = True
order=4
fes_u = L2(mesh, order=order, all_dofs_together=True)
fes_lam = L2(mesh, order=0)
fes = FESpace([fes_u, fes_lam])
(u, lam), (v, mu) = fes.TnT()

start = time.time()
# bilinear form
a = BilinearForm(fes, symmetric=True, geom_free=geom_free)
a += (grad(u)*grad(v)+lam*v+u*mu)*dx
a.Assemble()
if geom_free :
    proj = Projector(fes.FreeDofs(), True) # identity
    inva = CGSolver(mat=a.mat, pre=proj, maxsteps=50, printrates=False)
else :
    inva = a.mat.Inverse(fes.FreeDofs())

# right hand side
f = LinearForm(fes)
f += x*v*dx
f.Assemble()


gfu = GridFunction(fes)
gfu.vec.data = inva*f.vec
end = time.time()

print('elapsed time: ', end-start)
